#!/usr/bin/env python
#    This file is part of pyrsp

#    pyrsp is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrsp is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with pyrsp  If not, see <http://www.gnu.org/licenses/>.

# (C) 2014 by Stefan Marsiske, <s@ctrlc.hu>

from elftools.elf.elffile import ELFFile
from elftools.elf.sections import SymbolTableSection

class FCache():
    """ helper class to read out the source code lines
        from the debug section of an elf file """
    def __init__(self):
        self.fd = None
        self.name = None

    def get_src_lines(self, fname, start, end = None):
        """ returns the lines indexed from start to end of the file
            indicated by fname"""
        if not self.name or self.name != fname:
            if self.name:
                self.fd.close()
            try: self.fd = open(fname,'r')
            except: return ''
            self.name = fname

        self.fd.seek(0)
        line_ptr=0
        while line_ptr < start-1:
            self.fd.readline()
            line_ptr+=1
        if end and end>start:
            res = []
            while line_ptr<end:
                res.append(self.fd.readline())
                line_ptr+=1
            return ''.join(res).strip()
        else:
            return self.fd.readline().strip()

class ELF:
    def __init__(self, name):
        """ reads out the entry point, the .text segment addres, the
            symbol table, and the debugging information from the elf
            header.
        """
        self.name = name
        self.fcache = FCache()

        with open(self.name,'r') as stream:
            elffile = ELFFile(stream)

            # get entry point
            self.entry = elffile.header.e_entry

            # get text seg address
            section = elffile.get_section_by_name(b'.text')
            if not section:
                raise ValueError('No text segment found.')
            self.workarea = section.header['sh_addr']

            # init symbols
            section = elffile.get_section_by_name(b'.symtab')
            if not section:
                raise ValueError('No symbol table found. Perhaps this ELF has been stripped?')

            res = {}
            if isinstance(section, SymbolTableSection):
                for i in xrange(section.num_symbols()):
                    res[section.get_symbol(i).name]=(section.get_symbol(i).entry.st_value)
            self.symbols = res

            self.src_map = self.get_src_map(elffile)

    def get_src_map(self, elffile):
        """ builds a dictionary of the DWARF information, used to populate
            self.src_map

            returns a dictionary with either the address as key, or
            filename:lineno the values are respectively {addr, file,
            lineno, line} and {addr, line}
        """

        src_map = {}
        if not elffile.has_dwarf_info():
            raise ValueError("No DWARF info found")
        _dwarfinfo = elffile.get_dwarf_info()

        for cu in _dwarfinfo.iter_CUs():
            lineprogram = _dwarfinfo.line_program_for_CU(cu)

            cu_filename = lineprogram['file_entry'][0].name
            if len(lineprogram['include_directory']) > 0:
                dir_index = lineprogram['file_entry'][0].dir_index
                if dir_index > 0:
                    dir = lineprogram['include_directory'][dir_index - 1]
                else:
                    dir = '.'
                cu_filename = '%s/%s' % (dir, cu_filename)

            for entry in lineprogram.get_entries():
                state = entry.state
                if state:
                    fname = lineprogram['file_entry'][state.file - 1].name
                    line = self.fcache.get_src_lines(cu_filename, state.line)
                    src_map["%08x" % state.address] = {'file': fname, 'lineno': state.line, 'line': line}
                    try:
                        src_map["%s:%s" % (fname, state.line)].append({'addr': "%08x" % state.address, 'line': line})
                    except KeyError:
                        src_map["%s:%s" % (fname, state.line)]= [{'addr': "%08x" % state.address, 'line': line}]
        return src_map

    def get_bin(self):
        """ returns the .text and .rodata sections from elf file fname
        """
        with open(self.name,'r') as stream:
            elffile = ELFFile(stream)

            # get text seg address
            txt = elffile.get_section_by_name(b'.text')
            if not txt:
                raise ValueError('No text segment found.')
            return txt.data()



#!/usr/bin/env python

from construct import BitStruct, BitField, Padding, Enum, Flag

SCB_ICSR = 0xe000ed04
scb_icsr = BitStruct("scb_icsr",
                     Flag('NMIPENDSET'),
                     Padding(2),
                     Flag('PENDSVSET'),
                     Flag('PENDSVCLR'),
                     Flag('PENDSTSET'),
                     Flag('PENDSTCLR'),
                     Padding(1),
                     Flag('DEBUG'),
                     Flag('ISRPENDING'),
                     BitField('VECTPENDING', 10),
                     Flag('RETOBASE'),
                     Padding(2),
                     BitField('VECTACTIVE', 9),
                     )

SCB_SHCSR = 0xe000ed24
scb_shcsr = BitStruct("scb_shcsr",
                     Padding(13),
                      Flag('USGFAULTENA'),
                      Flag('BUSFAULTENA'),
                      Flag('MEMFAULTENA'),
                      Flag('SVCALLPENDED'),
                      Flag('BUSFAULTPENDED'),
                      Flag('MEMFAULTPENDED'),
                      Flag('USGFAULTPENDED'),
                      Flag('SYSTICKACT'),
                      Flag('PENDSVACT'),
                      Padding(1),
                      Flag('MONITORACT'),
                      Flag('SVCALLACT'),
                      Padding(3),
                      Flag('USGFAULTACT'),
                      Padding(1),
                      Flag('BUSFAULTACT'),
                      Flag('MEMFAULTACT'))

SCB_CFSR = 0xe000ed28
scb_cfsr = BitStruct("scb_cfsr",
                     Padding(6),
                     Flag("DIVBYZERO"),
                     Flag("UNALIGNED"),
                     Padding(4),
                     Flag("NOCP"),
                     Flag("INVPC"),
                     Flag("INVSTATE"),
                     Flag("UNDEFINSTR"),
                     Flag("BFARVALID"),
                     Padding(2),
                     Flag("STKERR"),
                     Flag("UNSTKERR"),
                     Flag("IMPRECISERR"),
                     Flag("PRECISERR"),
                     Flag("IBUSERR"),
                     Flag("MMARVALID"),
                     Padding(2),
                     Flag("MSTKERR"),
                     Flag("MUNSTKERR"),
                     Padding(1),
                     Flag("DACCVIOL"),
                     Flag("IACCVIOL"),
                     )

SCB_HFSR = 0xe000ed2c
scb_hfsr = BitStruct("scb_hfsr",
                     Flag("DEBUG_VT"),
                     Flag("FORCED"),
                     Padding(28),
                     Flag("VECTTBL"),
                     Padding(1),
                     )
SCB_MMFAR = 0xe000ed34
SCB_BFAR = 0xe000ed38

MPU_TYPER = 0xe000ed90
MPU_CR = 0xe000ed94
mpu_cr = BitStruct("mpu_cr",
                   Padding(29),
                   Flag("PRIVDEFENA"),
                   Flag("HFNMIENA"),
                   Flag("ENABLE"),
                   )

MPU_RNR = 0xe000ed98
mpu_rnr = BitStruct("mpu_rnr",
                    Padding(24),
                    BitField('REGION', 8))

MPU_RBAR = 0xe000ed9c
mpu_rbar = BitStruct("mpu_rbar",
                     BitField('ADDR', 27),
                     Flag('VALID'),
                     BitField('REGION', 4))

MPU_RASR = 0xe000eda0
mpu_rasr = BitStruct("mpu_rasr",
                     Padding(3),
                     Flag("XN"),
                     Padding(1),
                     #BitField("AP", 3),
                     Enum(BitField("AP", 3),
                          No_access = 0,
                          RW_No_access = 1,
                          RW_RO = 2,
                          RW = 3,
                          RO_No_access = 5,
                          RO = 6,
                          INV7 = 7),
                     Padding(2),
                     #BitField("TEX", 3), #Flag("S", 1), #Flag("C", 1), #Flag("B", 1),
                     Enum(BitField("TEXSCB", 6),
                          UNSET   =(0b000000),
                          FLASH_RAM   =(0b000010),
                          INTERNAL_RAM=(0b000110),
                          EXTERNAL_RAM=(0b000111),
                          PERIPHERIALS=(0b000101),),
                     BitField("SRD", 8),
                     Padding(2),
                     #BitField("SIZE", 5),
                     Enum(BitField("SIZE", 5),
                         unset   =(0b00000), invalid1=(0b00001), invalid2=(0b00010), invalid3=(0b00011),
                         _32B    =(0b00100), _64B    =(0b00101), _128B   =(0b00110), _256B   =(0b00111),
                         _512B   =(0b01000), _1KB    =(0b01001), _2KB    =(0b01010), _4KB    =(0b01011),
                         _8KB    =(0b01100), _16KB   =(0b01101), _32KB   =(0b01110), _64KB   =(0b01111),
                         _128KB  =(0b10000), _256KB  =(0b10001), _512KB  =(0b10010), _1MB    =(0b10011),
                         _2MB    =(0b10100), _4MB    =(0b10101), _8MB    =(0b10110), _16MB   =(0b10111),
                         _32MB   =(0b11000), _64MB   =(0b11001), _128MB  =(0b11010), _256MB  =(0b11011),
                         _512MB  =(0b11100), _1GB    =(0b11101), _2GB    =(0b11110), _4GB    =(0b11111),
                     ),
                     Flag("Enabled"))



#/usr/bin/env python2
import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()

setup(
    name = "pyrsp",
    version = "0.3.1",
    author = "Stefan Marsiske",
    author_email = "s@ctrlc.hu",
    description = ("simple GDB remote serial protocol wrapper"),
    license = "AGPLv3",
    keywords = "GDB remote debugging JTAG embedded scripting",
    url = "https://github.com/stef/pyrsp",
    packages = ['pyrsp'],
    entry_points = {
       'console_scripts': [
          'pyrsp = pyrsp.rsp:main',
          ],
       },
    long_description=read('README.org'),
    install_requires = ("pyelftools", "pyserial"),
    classifiers = ["Development Status :: 4 - Beta",
                   "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
                   ],
)



#!/usr/bin/env python
#    This file is part of pyrsp

#    pyrsp is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrsp is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with pyrsp  If not, see <http://www.gnu.org/licenses/>.

# (C) 2014 by Stefan Marsiske, <s@ctrlc.hu>

def pack(data):
    """ formats data into a RSP packet """
    for a, b in [(x, chr(ord(x) ^ 0x20)) for x in ['}','*','#','$']]:
        data = data.replace(a,'}%s' % b)
    return "$%s#%02X" % (data, (sum(ord(c) for c in data) % 256))

def unpack(pkt):
    """ unpacks an RSP packet, returns the data"""
    if pkt[0]!='$' or pkt[-3]!='#':
        raise ValueError('bad packet')
    if (sum(ord(c) for c in pkt[1:-3]) % 256) != int(pkt[-2:],16):
        raise ValueError('bad checksum')
    pkt = pkt[1:-3]
    return pkt

def unhex(data):
    """ takes a hex encoded string and returns the binary representation """
    return ''.join(chr(int(x,16)) for x in split_by_n(data,2))

def switch_endian(data):
    """ byte-wise reverses a hex encoded string """
    return ''.join(reversed(list(split_by_n( data ,2))))

def split_by_n( seq, n ):
    """A generator to divide a sequence into chunks of n units.
       src: http://stackoverflow.com/questions/9475241/split-python-string-every-nth-character"""
    while seq:
        yield seq[:n]
        seq = seq[n:]

def hexdump(data, ptr=0):
    """ returns data formatted as a hexdump """
    return "\t%s" % '\n\t'.join(["%08x | %s %s" % ((i*32)+ptr,
                                                   ' '.join([''.join(["%02x" % ord(c) for c in word])
                                                             for word in split_by_n(line,8)]),
                                                   ''.join([c if c.isalnum() else '.' for c in line]))
                                 for i, line in enumerate(split_by_n(data,32))])




#!/usr/bin/env python
#    This file is part of pyrsp

#    pyrsp is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrsp is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with pyrsp  If not, see <http://www.gnu.org/licenses/>.

# (C) 2014 by Stefan Marsiske, <s@ctrlc.hu>

import os, time, sys, struct, socket
activate_this = os.path.dirname(__file__)+'/../env/bin/activate_this.py'
if os.path.exists(activate_this):
    execfile(activate_this, dict(__file__=activate_this))

import serial
from pyrsp.utils import hexdump, pack, unpack, unhex, switch_endian, split_by_n
from pyrsp.elf import ELF
from binascii import hexlify

def Debugger(*args, **kwargs):
    if os.path.exists(args[0]):
        return BlackMagic(*args,**kwargs)
    try:
        int(args[0])
    except:
        print "cannot access", args[0]
        sys.exit(0)
    return STlink2(*args, **kwargs)

class BlackMagic(object):
    def __init__(self, port):
        self.__dict__['port'] = serial.Serial(port, 115200, timeout=1)

    def setup(self, rsp):
        rsp.send('qRcmd,737764705f7363616e')
        pkt=rsp.readpkt()
        while pkt!='OK':
            if pkt[0]!='O':
                raise ValueError('not O: %s' % pkt)
            if rsp.verbose:
                print unhex(pkt[1:-1])
            pkt=rsp.readpkt()
        rsp.fetchOK('vAttach;1','T05')

    def write(self, data):
        return self.port.write(data)

    def read(self, size=1):
        return self.port.read(size)

    def close(self, rsp):
        rsp.fetchOK('D')
        self.port.close()

class STlink2(object):
    def __init__(self, port):
        self.__dict__['port'] = socket.socket( socket.AF_INET,socket.SOCK_STREAM)
        self.port.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.port.settimeout(1)
        self.port.connect(('localhost',int(port)))

    def setup(self, rsp):
        pass

    def write(self, data):
        i = 0
        while(i<len(data)):
            i += self.port.send(data[i:])
        return i

    def read(self):
        try:
            return self.port.recv(1)
        except:
            return ''

    def close(self):
        self.port.close()

class RSP(object):
    def __init__(self, port, elffile=None, verbose=False):
        """ read the elf file if given by elffile, connects to the
            debugging device specified by port, and initializes itself.
        """
        self.registers = self.arch['regs']
        self.__dict__['br'] = {}
        self.__dict__['verbose'] = verbose
        # open serial connection
        self.__dict__['port'] = Debugger(port) #serial.Serial(port, 115200, timeout=1)
        # parse elf for symbol table, entry point and work area
        self.__dict__['elf'] = ELF(elffile) if elffile else None
        if verbose and self.elf:
            print "work area: 0x%x" % self.elf.workarea
            print "entry: 0x%x" % self.elf.entry

        # check for signal from running target
        tmp = self.readpkt(timeout=1)
        if tmp: print 'helo', tmp

        self.send('qSupported')
        feats = self.readpkt()
        if feats:
            self.feats = dict((ass.split('=') if '=' in ass else (ass,None) for ass in feats.split(';')))

        # attach
        self.connect()

    def connect(self):
        pass

    def send(self, data, retries=50):
        """ sends data via the RSP protocol to the device """
        self.port.write(pack(data))
        res = None
        while not res:
            res = self.port.read()
        discards = []
        while res!='+' and retries>0:
            discards.append(res)
            #self.port.write(pack(data))
            retries-=1
            res = self.port.read()
        if len(discards)>0 and self.verbose: print 'send discards', discards
        if retries==0:
            raise ValueError("retry fail")
        #print 'sent', data

    def readpkt(self, timeout=0):
        """ blocks until it reads an RSP packet, and returns it's
            data"""
        c=None
        discards=[]
        if timeout>0:
            start = time.time()
        while(c!='$'):
            if c: discards.append(c)
            c=self.port.read()
            if timeout>0 and start+timeout < time.time():
                return
        if len(discards)>0 and self.verbose: print 'discards', discards
        res=[c]

        while True:
            res.append(self.port.read())
            if res[-1]=='#' and res[-2]!="'":
                res.append(self.port.read())
                res.append(self.port.read())
                try:
                    res=unpack(''.join(res))
                except:
                    self.port.write('-')
                    res=[]
                    continue
                self.port.write('+')
                #print "read", res
                return res

    def store(self, data, addr=None):
        """ stores data at addr if given otherwise at beginning of
            .text segment aka self.elf.workarea"""
        if addr==None:
            addr=self.elf.workarea
        for pkt in split_by_n(hexlify(data), int(self.feats['PacketSize'],16) - 20):
            pktlen = len(pkt)/2
            self.fetchOK('M%x,%x:%s' % (addr, pktlen, pkt))
            addr+=pktlen

    def __getslice__(self, i, j):
        return self.dump(j-i,i)

    def __setitem__(self, i,val):
        self.store(val,i)

    def __getattr__(self, name):
        if name not in self.__dict__ or not self.__dict__[name]:
            if name in self.regs.keys():
                return self.regs[name]
        if name in self.__dict__.keys():
            return self.__dict__[name]
        else:
            raise AttributeError, name

    def dump(self, size, addr = None):
        """ dumps data from addr if given otherwise at beginning of
            .text segment aka self.elf.workarea"""
        if addr==None:
            addr=self.elf.workarea
        rd = []
        i=0
        bsize = int(self.feats['PacketSize'], 16)
        while(i<size):
            bsize = bsize if i+bsize<size else size - i
            self.send('m%x,%x' % (addr+i, bsize))
            pkt=self.readpkt()
            #print pkt
            rd.append(unhex(pkt))
            i+=bsize
        return ''.join(rd)

    def fetch(self,data):
        """ sends data and returns reply """
        self.send(data)
        return self.readpkt()

    def fetchOK(self,data,ok='OK'):
        """ sends data and expects success """
        res = self.fetch(data)
        if res!=ok: raise ValueError(res)

    def set_reg(self, reg, val):
        """ sets value of register reg to val on device """
        if isinstance(val, str):
            self.regs[reg]=val
        if isinstance(val, int):
            self.regs[reg]='%x' % val
        self.fetchOK("G%s" % ''.join([switch_endian(self.regs[r]) for r in self.registers if r in self.regs]))

    def refresh_regs(self):
        """ loads and caches values of the registers on the device """
        self.send('g')
        if self.arch['endian']:
            self.regs=dict(zip(self.registers,(switch_endian(reg) for reg in split_by_n(self.readpkt(),self.arch['bitsize']>>2))))
        else:
            self.regs=dict(zip(self.registers,split_by_n(self.readpkt(),self.arch['bitsize']>>2)))

    def dump_regs(self):
        """ refreshes and dumps registers via stdout """
        self.refresh_regs()
        print ' '.join(["%s:%s" % (r, self.regs.get(r)) for r in self.registers])

    def get_thread_info(self):
        tid = None
        tmp = self.fetch('qC')
        if tmp.startswith("QC"):
            tid=tmp[2:].strip()
        extra = unhex(self.fetch('qThreadExtraInfo,%s' % tid))
        tids = []
        tmp = self.fetch('qfThreadInfo')
        while tmp != 'l':
            if not tmp.startswith('m'):
                raise ValueError('invalid qThreadInfo response')
            tids.extend(tmp[1:].split(','))
            tmp = self.fetch('qsThreadInfo')
        return (tid, extra, tids)

    def run(self, start=None):
        """ sets pc to start if given or to entry address from elf header,
            passes control to the device and handles breakpoints
        """
        if not start:
            entry = "%08x" % self.elf.entry
        else:
            entry = "%08x" % (self.elf.symbols[start] & ~1)
        if self.verbose: print "set new pc: @test (0x%s)" % entry,
        self.set_reg('pc', entry)
        if self.verbose: print 'OK'

        if self.verbose: print "continuing"
        sig = self.fetch('c')
        while sig in ['T05', 'S05']:
            self.handle_br()
            sig = self.fetch('c')

        if sig!='T0B': print 'strange signal', sig
        if hasattr(self, 'checkfault'):
            self.checkfault()
        else:
            src_line = self.get_src_line(int(self.regs['pc'],16 - 1))
            if src_line:
                print "0 %s:%s %s" % (src_line['file'], src_line['lineno'], src_line['line'])
            else:
                print 'pc', self.regs['pc']
            src_line = self.get_src_line(int(self.regs['lr'],16) -3)
            if src_line:
                print "1 %s:%s %s" % (src_line['file'], src_line['lineno'], src_line['line'])
            else:
                print 'lr', self.regs['lr']
            self.dump_regs()

        self.port.close(self)
        sys.exit(0)

    def handle_br(self):
        """ dumps register on breakpoint/signal, continues if unknown,
            otherwise it calls the appropriate callback.
        """
        print
        self.dump_regs()
        if not self.regs['pc'] in self.br:
            print "unknown break point passed"
            self.dump_regs()
            return
        if self.verbose:
            print 'breakpoint hit:', self.br[self.regs['pc']]['sym']
        self.br[self.regs['pc']]['cb']()

    def set_br(self, sym, cb, quiet=False):
        """ sets a breakpoint at symbol sym, and install callback cb
            for it
        """
        addr = self.elf.symbols.get(sym)
        if not addr:
            print "unknown symbol: %s, ignoring request to set br" % sym
            return
        addr = "%08x" % (addr & ~1)
        if addr in self.br:
            print "warn: overwriting breakpoint at %s" % sym
            self.br[addr]={'sym': sym,
                           'cb': cb,
                           'old': self.br[addr]['old']}
        else:
            self.br[addr]={'sym': sym,
                           'cb': cb,
                           'old': unhex(self.fetch('m%s,2' % addr))}
        #self.fetch('Z0,%s,2' % addr)
        tmp = self.fetch('X%s,2:\xbe\xbe' % addr)
        if self.verbose and not quiet:
            print "set break: @%s (0x%s)" % (sym, addr), tmp

    def del_br(self, addr, quiet=False):
        """ deletes breakpoint at address addr """
        sym = self.br[addr]['sym']
        #self.fetch('z0,%s,2' % addr)
        tmp = self.fetch('X%s,2:%s' % (addr, self.br[addr]['old']))
        if self.verbose and not quiet: print "clear breakpoint: @%s (0x%s)" % (sym, addr), tmp
        del self.br[addr]

    def finish_cb(self):
        """ final breakpoint, if hit it deletes all breakpoints,
            continues running the cpu, and detaches from the debugging
            device
        """
        # clear all breaks
        for br in self.br.keys()[:]:
            self.del_br(br)
        if self.verbose:
            print "continuing and detaching"
        # leave in running state
        self.send('c')
        sys.exit(0)

    def get_src_line(self, addr):
        """ returns the source-code line associated with address addr
        """
        i = 0
        src_line = None
        while not src_line and i<1023:
            src_line = self.elf.src_map.get("%08x" % (addr - i))
            i+=2
        return src_line

    def step_over_br(self):
        sym = self.br[self.regs['pc']]['sym']
        cb  = self.br[self.regs['pc']]['cb']
        self.del_br(self.regs['pc'], quiet=True)
        sig = self.fetch('s')
        if sig in ['T05', 'T0B']:
            self.set_br(sym, cb, quiet=True)
        else:
            print 'strange signal while stepi over br, abort'
            sys.exit(1)

    def dump_cb(self):
        """ rsp_dump callback, hit if rsp_dump is called. Outputs to
            stdout the source line, and a hexdump of the memory
            pointed by $r0 with a size of $r1 bytes. Then it resumes
            running.
        """
        src_line = self.get_src_line(int(self.regs['lr'],16) - 3)
        if src_line:
            print "%s:%s %s" % (src_line['file'], src_line['lineno'], src_line['line'])

        res_size = int(self.regs['r1'],16)
        if res_size <= 1024: # for sanity
            ptr = int(self.regs['r0'],16)
            res = unhex(self.fetch('m%x,%x' % (ptr, res_size)))
            print hexdump(res, ptr)

        self.step_over_br()

    def load(self, verify):
        """ loads binary belonging to elf to beginning of .text
            segment (alias self.elf.workarea), and if verify is set read
            it back and check if it matches with the uploaded binary.
        """
        if self.verbose: print 'load %s' % self.elf.name
        buf = self.elf.get_bin()
        self.store(buf)

        if verify:
            if self.verbose: print "verify test",
            if not self.dump(len(buf)) == buf:
                raise ValueError("uploaded binary failed to verify")
            if self.verbose: print 'OK'

    def call(self, start=None, finish='rsp_finish', dump='rsp_dump', verify=True):
        """
        1. Loads the '.bin' file given by self.elf into the device
           at the workarea (.text seg) of the device.
        2. Using the '.elf' file it sets a breakpoint on the function
           specified by rsp_finish and rsp_dump,
        3. and starts execution at the function specified by start or elf e_entry.
        4. After the breakpoint of rsp_dump is hit, r1 bytes are dumped
           from the buffer pointed to by r0.
        5. After the breakpoint of rsp_finish is hit, it removes all
           break points, and detaches
        """

        self.refresh_regs()
        self.load(verify)
        self.set_br(finish, self.finish_cb)
        self.set_br('rsp_detach', self.finish_cb)
        self.set_br(dump, self.dump_cb)
        self.run(start)

    def test(self):
        # show current regs
        self.dump_regs()

        # test write
        self.fetchOK('X%08x,0' % self.elf.workarea)

        # reset workspace area
        self.store('\x00' * 2048)

        # verify workspace area empty
        if self.dump(2048) != '\x00' * 2048:
            raise ValueError('cannot erase work area')

from cortexhwregs import *
class CortexM3(RSP):
    def __init__(self, *args,**kwargs):

        self.arch = {'regs': ["r0", "r1", "r2", "r3", "r4", "r5", "r6", "r7", "r8",
                              "r9", "r10", "r11", "r12", "sp", "lr", "pc",
                              "xpsr", "msp", "psp", "special"],
                     'endian': True,
                     'bitsize': 32}
        super(CortexM3,self).__init__(*args, **kwargs)

    def get_thread_info(self):
        return

    def getreg(self,size,ptr):
        tmp = self.fetch('m%x,%x' % (ptr, size))
        return unhex(switch_endian(tmp))

    def printreg(self, reg):
        return [n if type(v)==bool and v==True else (n,v if n!='ADDR' else hex(v<<5)) for n,v in reg.items() if v]

    def dump_mpu(self):
        print 'mpu_cr', self.printreg(mpu_cr.parse(self.getreg(4, MPU_CR)))
        for region in xrange(8):
            self.store(struct.pack("<I", region), MPU_RNR)
            print region,
            print self.printreg(mpu_rbar.parse(self.getreg(4, MPU_RBAR))),
            print self.printreg(mpu_rasr.parse(self.getreg(4, MPU_RASR)))

    def checkfault(self):
        # impl check, only dumps now.
        #sig = self.fetch('s')
        #print 'sig', sig
        self.dump_mpu()
        print 'hfsr=', self.printreg(scb_hfsr.parse(self.getreg(4, SCB_HFSR)))
        print 'icsr=', self.printreg(scb_icsr.parse(self.getreg(4, SCB_ICSR)))
        print 'shcsr=', self.printreg(scb_shcsr.parse(self.getreg(4, SCB_SHCSR)))
        print 'cfsr=', self.printreg(scb_cfsr.parse(self.getreg(4, SCB_CFSR)))
        print 'MMFAR=', hex(struct.unpack(">I", self.getreg(4, SCB_MMFAR))[0])
        print 'BFAR=', hex(struct.unpack(">I", self.getreg(4, SCB_BFAR))[0])
        print
        src_line = self.get_src_line(struct.unpack(">I", self.getreg(4, int(self.regs['sp'],16) + 24))[0])
        if src_line:
            print "%s:%s %s" % (src_line['file'], src_line['lineno'], src_line['line'])
        else:
            print 'sp', format(struct.unpack(">I", self.getreg(4, int(self.regs['sp'],16) + 24))[0], '08x')

        self.port.close(self)
        sys.exit(0)

    def connect(self):
        """ attaches to blackmagic jtag debugger in swd mode """
        # ignore redundant stuff
        tmp = self.readpkt(timeout=1)
        while(tmp):
            tmp = self.readpkt(timeout=1)
        # enable extended mode
        self.fetchOK('!')

        # setup registers TODO
        # registers should be parsed from the output of, see target.xml
        #print self.fetch('qXfer:features:read:target.xml:0,3fb')
        #print self.fetch('Xfer:features:read:target.xml:3cf,3fb')
        #print self.fetch('qXfer:memory-map:read::0,3fb')
        #print self.fetch('qXfer:memory-map:read::364,3fb')

        self.port.setup(self)

        addr=struct.unpack(">I", self.getreg(4, 0x0800000c))[0] - 1
        self.br[format(addr, '08x')]={'sym': "FaultHandler",
                             'cb': self.checkfault}
        tmp = self.fetch('Z1,%s,2' % format(addr, 'x'))
        if tmp== 'OK':
            print "set break: @%s (0x%s)" % ('FaultHandler', format(addr, 'x')), tmp
            return

        # vector_catch enable hard int bus stat chk nocp mm reset
        self.send('qRcmd,766563746f725f636174636820656e61626c65206861726420696e742062757320737461742063686b206e6f6370206d6d207265736574')
        pkt=self.readpkt()
        while pkt!='OK':
            if pkt[0]!='O':
                raise ValueError('not O: %s' % pkt)
            if self.verbose:
                print unhex(pkt[1:-1])
            pkt=self.readpkt()

class AMD64(RSP):
    def __init__(self, *args,**kwargs):
        self.arch = {'regs': ["rax", "rbx", "rcx", "rdx", "rsi", "rdi", "rbp", "rsp",
                              "r8", "r9", "r10", "r11", "r12", "r13", "r14", "r15",
                              "rip", "eflags", "cs", "ss", "ds", "es", "fs", "gs",
                              "st0", "st1", "st2", "st3", "st4", "st5", "st6", "st7",
                              "fctrl", "fstat", "ftag", "fiseg", "fioff", "foseg", "fooff", "fop",
                              "xmm0", "xmm1", "xmm2", "xmm3", "xmm4", "xmm5", "xmm6", "xmm7",
                              "xmm8", "xmm9", "xmm10", "xmm11", "xmm12", "xmm13", "xmm14", "xmm15",
                              "mxcsr",],
                     'endian': False,
                     'bitsize': 64}
        super(AMD64,self).__init__(*args, **kwargs)

class i386(RSP):
    # TODO gdb sends qSupported:multiprocess+;xmlRegisters=i386;qRelocInsn+
    def __init__(self, *args,**kwargs):
        self.arch = {'regs': ["eax", "ecx", "edx", "ebx", "esp", "ebp", "esi", "edi",
                             "eip", "eflags", "cs", "ss", "ds", "es", "fs", "gs",
                             "st0", "st1", "st2", "st3", "st4", "st5", "st6", "st7",
                             "fctrl", "fstat", "ftag", "fiseg", "fioff", "foseg", "fooff", "fop",
                             "xmm0", "xmm1", "xmm2", "xmm3", "xmm4", "xmm5", "xmm6", "xmm7",
                             "mxcsr"],
                    'endian': False,
                    'bitsize': 32}
        super(i386,self).__init__(*args, **kwargs)

archmap={'amd64': AMD64, "i386": i386, "cortexm3": CortexM3}

def main():
    # parse arch from cmdline
    arch=i386
    for i, arg in enumerate(sys.argv):
        if arg in archmap:
            arch=archmap[arg]
            del sys.argv[i]
            break
    # argv[1] should be the file to the debugger device, e.g: /dev/ttyACM0
    # argv[2] can be the elf file
    if len(sys.argv)<2:
        print "%s [<%s>] <serial interface> [<elf file>]" % (sys.argv[0],
                                                             '|'.join(archmap.keys()))
        sys.exit(1)

    elffile=sys.argv[2] if len(sys.argv)>2 else None

    rsp = arch(sys.argv[1], elffile, verbose=False)

    if elffile:
        rsp.call()
    else:
        print hexdump(rsp.dump(2048, 0),0)
        rsp.dump_regs()
        print rsp.get_thread_info()
        rsp.send('c')

if __name__ == "__main__":
    main()
